/* Sound player

   Copyright (C) 1997 by Woo-jae Jung 

  Original Hangul output routine Author
       Choi Jun-Ho(junker@jazz.ac.kr)
       Narae, CS Dept, SNU
*/
   
#include <X11/Xlib.h>

#include <stdio.h>

#define MAX_LINE    256

// default hangul font
// if your machine is Solaris 2.4 and above, trying this for default font..
// #define NX_DEFAULTFONT \
//           "-sun-gothic-medium-r-normal--12-120-75-75-c-120-ksc5601.1987-0"*/

// this works almost all cases... NX_DEFALUTFONT must exist!
//#define NX_DEFAULTFONT \
//       "-daewoo-gothic-medium-r-normal--16-120-100-100-c-160-ksc5601.1987-0"

char *hanfontname=
       "-daewoo-gothic-medium-r-normal--16-120-100-100-c-160-ksc5601.1987-0";

typedef struct _NXFont{
  int ascent;
  char* fontname;
  XFontStruct* font;
  XID fid;
  struct _NXFont* next; /* linked list */
} NXFont;

#define ABS(x)    ((x)>0?(x):-(x))

/* globals */
static NXFont defaultfont;

/****** utils *******/
/* EUC_KR2WC()
   
   convert EUC-KR(KSC-5601) char string to XChar2b struct.
   corresfonding XLFD *-KSC5601.1987-0
   */
static int EUC_KR2WC(const char *str,XChar2b *buf,int n)
{
  int sp, xcp;

  for (sp = 0, xcp = 0; sp < n;) {
    /* in case of 8bit 2b-char */
    buf[xcp].byte1 = str[sp] - 0x80;
    buf[xcp].byte2 = str[sp+1] - 0x80;
    sp += 2, xcp++;
  }
  return xcp;
}

/* ConvEUC_KR7()

    convert 7-bit KSC-5601 code to 8-bit KSC-5601 code
      ESC-$-(-C (0x1b 0x24 0x28 0x43) turns on 7-bit mode
      ESC-(-B (0x1b 0x28 0x42) turns off 7-bit mode
      in 7-bit mode, char must be in 0x21~0x7e
      MSB of char in 7-bit mode is set
*/
static int ConvEUC_KR7(const char *string,char *deststr,int len)
{
  int i, j;
  int is_7bit;

  i=0; j=0; is_7bit=0;

  while(i<len){
    if (string[i]==0x1b){
      if (i+4<len &&
          string[i+1]==0x24 &&
          (string[i+2]==0x28 || string[i+2]==0x29) &&
          string[i+3]==0x43){
        is_7bit=1;  /* turns on 7bit mode */
        i+=4;       /* jump to next char */
        continue;
      }
      if (i+3<len &&
          string[i+1]==0x28 &&
          string[i+2]==0x42){
        is_7bit=0; /* turns off 7bit mode */
        i+=3;
        continue;
      }
      /* normal ESC sequence */
    }
    if (is_7bit==1 && (string[i]>=0x21 && string[i]<=0x7e))
      /* set MSB */
      deststr[j++]=string[i] | 0x80;
    else
      deststr[j++]=string[i];
    i++;
  }

  return j;
}


/* QueryMatchingFont()

   following to <ascent>, find Korean font and returns XFontStruct*.
   */
static XFontStruct *QueryMatchingFont(Display *,int )
{
  NXFont *selfont;

  /* set default font */
  selfont=&defaultfont;

  if (selfont==NULL)
    return (&defaultfont)->font;

  return selfont->font;
}

void Koreaninit(void)
{
  extern Display *qt_xdisplay();

  defaultfont.font=XLoadQueryFont(qt_xdisplay(),hanfontname);
}

extern "C"
{
int __XDrawString__(Display *display,Drawable d,GC gc,
		    int x,int y,const char *str,int len);
}

int XDrawString(Display *display,Drawable d,GC gc,
		int x,int y,const char *str,int len)
{
  int n;
  int clen;
  char *p, *cbuf;
  char cbuf_tmp[MAX_LINE];
  XChar2b buf[MAX_LINE];
  XFontStruct *efont,*hfont;

  {
    Font engfid;
    XGCValues gcv;

    XGetGCValues(display, gc, GCFont, &gcv);
    engfid=gcv.font;
    efont=XQueryFont(display, engfid);
  }

  /* get hangul font according to english font */
  hfont=QueryMatchingFont(display,efont->ascent);

  /* convert chars if they are in 7-bit mode */
  clen=ConvEUC_KR7(str, cbuf_tmp, len); 

  cbuf=cbuf_tmp;
  while (clen > 0) {
    if (*cbuf & 0x80) { /* hangul chars */
      for (p = cbuf; clen > 0 && (*p & 0x80); p += 2, clen -= 2)
        ;
      n = EUC_KR2WC(cbuf, buf, p-cbuf);
      XSetFont(display,gc,hfont->fid);
      XDrawString16(display, d, gc, x, y, buf, n);
      x+=XTextWidth16(hfont, buf, n);
    }
    else { /* english chars */
      for (p = cbuf; clen > 0 && !(*p & 0x80); p++, clen--)
        ;
      XSetFont(display,gc,efont->fid);
      __XDrawString__(display, d, gc, x, y, cbuf, p-cbuf); 
      x+=XTextWidth(efont, cbuf, p-cbuf);
    }
    cbuf = p;
  }

  return 0;
}
